
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>

#include "main.h"
#include "data.h"

#define DEBUG 		0
#include "debug.h"


static int cmd_set_playername(CLIENT *client, char *data) {
	DBG_MSG(3, "cmd_set_playername: %s", data);
	if (client && data && data[0] != '\0') {
		if (client->name)
			free(client->name);
		client->name = strdup(data);
		
		if (cfg.verbose) {
			printf("Player with socket %d is now known as \"%s\"\n", client->fd, client->name);
		}
		
		return 1;
	}
	return 0;
}

static int cmd_join(CLIENT *client, char *data) {
	char *msg;
	const char *cmd_p = "PLAYER ";
	const char *cmd_c = "COLOR ";
	CLIENT *c;
	int i;

	if (!client || !client->name || !data) {
		return 0;
	}
	
	switch (connect_to_game(client, data)) {
		case 1: /* connected */
			msg = malloc_c(strlen(client->name) + strlen(cmd_p) + 20);
			sprintf(msg, "%s%d %s\n", cmd_p, client->fd, client->name);
			send_to_all(client->game, msg, client);
			free(msg);
			
			if (cfg.verbose) {
				printf("Player \"%s\" (socket %d) has joined the game \"%s\".\n",
						client->name ? client->name : "", client->fd,
						data);
			}
			break;

		case 2: /* connected to new */
			if (cfg.verbose) {
				printf("Player \"%s\" (socket %d) has joined NEW game \"%s\".\n",
						client->name ? client->name : "", client->fd,
						data);
			}
			break;

		default: /* error */
			if (cfg.verbose) {
				printf("Player \"%s\" (socket %d) can't join the game \"%s\".\n",
						client->name ? client->name : "", client->fd,
						data);
			}
			return 0;
	}

	/* send player list */
	for (i = 0; client->game->player && i < client->game->player_size; i++) {
		if ((c = (CLIENT *)client->game->player[i])) {
			/* send player */
			msg = malloc_c(strlen(c->name) + strlen(cmd_p) + 20);
			sprintf(msg, "%s%d %s\n", cmd_p, c->fd, c->name);
			DBG_MSG(10, "cmd_join: Sending: \"%s\"", msg);
			write(client->fd, msg, strlen(msg));
			free(msg);
			
			/* send color */
			msg = malloc_c(strlen(c->name) + strlen(cmd_p) + 20);
			sprintf(msg, "%s%d %d %s\n", cmd_c, c->fd, c->color, c->name);
			DBG_MSG(10, "cmd_join: Sending: \"%s\"", msg);
			write(client->fd, msg, strlen(msg));
			free(msg);
		}
	}

	return 1;
}

static int cmd_send_msg(CLIENT *client, char *data) {
	char *buf = NULL;
	char *cmd = "MSG";
	
	if (!data)
		return 0;
	
	if (client->game) {
		if (cfg.verbose)
			printf("Player \"%s\" (socket %d): \"%s\"\n",
					client->name ? client->name : "", client->fd,
					data);

		buf = (char *) malloc_c(strlen(cmd) + strlen(client->name) + strlen(data) + 20);
		sprintf(buf, "%s %d %s: %s\n", cmd, client->fd, client->name, data);
		send_to_all(client->game, buf, NULL);

		free(buf);
		return 1;
	} else {
		DBG_MSG(8, "cmd_send_msg: Hrac \"%s\" (socket %d) neni ve hre", client->name, client->fd);
		return 0;
	}
}

static int cmd_close(CLIENT *client, char *data) {
	if (cfg.verbose)
		printf("Player \"%s\" (socket %d) has left.\n",
				client->name ? client->name : "", client->fd);
	leave_client(client);
	return 1;
}

static int cmd_color(CLIENT *client, char *data) {
	int color, i;
	CLIENT *c;
	char *buf;
	
	if (sscanf(data, "%d", &color) != 1)
		return 0;
	
	
	for (i = 0; client->game->player && i < client->game->player_size; i++) {
		if ((c = (CLIENT *)client->game->player[i])
				&& c->color == color) {
			if (cfg.verbose)
				printf("Player \"%s\" (socket %d) wants allerady associated color %d.\n",
						client->name ? client->name : "", client->fd, color);
			return 0;
		}
	}
	if (cfg.verbose)
		printf("Player \"%s\" (socket %d) has color %d.\n",
				client->name ? client->name : "", client->fd, color);

	client->color = color;
	
	buf = (char *) malloc_c(strlen(client->name) + 10);
	sprintf(buf, "COLOR %d %d %s\n", client->fd, client->color, client->name);
	send_to_all(client->game, buf, NULL /*client*/);
	free(buf);
	return 1;
}


static int cmd_turn (CLIENT *client, char *data) {
	char *buf = NULL;
	char *cmd = "TURN ";
	
	if (!data)
		return 0;
	
	if (client->game) {
		if (cfg.verbose)
			printf("Player \"%s\" (socket %d): TURN: \" %s\"\n",
					client->name ? client->name : "", client->fd,
					data);

		buf = (char *) malloc_c(strlen(cmd) + strlen(data) + 2);
		strcpy(buf, cmd);
		strcat(buf, data);
		strcat(buf, "\n");
		
		send_to_all(client->game, buf, client);

		free(buf);
		return 1;
	} else {
		DBG_MSG(8, "cmd_turn: Hrac \"%s\" (socket %d) neni ve hre", client->name, client->fd);
		return 0;
	}
}

static int cmd_start (CLIENT *client, char *data) {
	if (client->game) {
		if (cfg.verbose)
			printf("Player \"%s\" (socket %d): START\n",
					client->name ? client->name : "", client->fd);
		send_to_all(client->game, "START\n", client);
		return 1;
	} else {
		DBG_MSG(8, "cmd_start: Hrac \"%s\" (socket %d) neni ve hre", client->name, client->fd);
		return 0;
	}
}


/*****************************************************************************
 * 	parse cmd
 ****************************************************************************/

static struct {
	char *cmd;
	int (*fn)(CLIENT *, char *);
} command[] = {
	{ "PLAYER",	cmd_set_playername },
	{ "JOIN",	cmd_join },
	{ "MSG",	cmd_send_msg },
	{ "CLOSE",	cmd_close },
	{ "COLOR",	cmd_color },
	{ "TURN",	cmd_turn },
	{ "START",	cmd_start },
	{ NULL, NULL }
};

int parse_cmd(CLIENT *client, char *data) {
	char *arg, *p_data = data, *p;
	int i;

	if ((p = strchr(data, '\n')))
		*p = '\0';
	if ((p = strchr(data, '\r')))
		*p = '\0';

	DBG_MSG(5, "parse_cmd: parsing command \"%s\"", data);

	arg = strchr(data, ' ');
	if (arg && *arg) {
		if (*arg)
			*arg++ = '\0';
		else
			*arg = '\0';
	} else {
		arg = NULL;
	}
	
	DBG_MSG(6, "parse_cmd: command is \"%s\", argument is \"%s\"", data, arg);

	for (i = 0; command[i].cmd; i++) {
		while(*p_data && *p_data != ' ' && (*p_data = toupper(*p_data)))
				p_data++;
		if (strcmp(command[i].cmd, data) == 0) {
			DBG_MSG(4, "parse_cmd: launching command \"%s(%s)\"", command[i].cmd, arg);
			return (*command[i].fn)(client, arg);
		}
	}
	return 0;
}

