#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "main.h"
#include "socket.h"
#include "data.h"

#define DEBUG	0
#include "debug.h"

GAME_LIST *game_list = NULL;

CLIENT *create_clients(int count) {
	CLIENT *client;
	
	if (count < 0) {
		fprintf(stderr, "create_clients: count < 0 !\n");
		exit(-1);
	}
	
	client = (CLIENT *) malloc_c(sizeof(CLIENT) * count);
	bzero(client, sizeof(CLIENT) * count);

	return client;
}

GAME_LIST *create_game_list(int size) {
	GAME_LIST *gl;

	if (size < 0) {
		fprintf(stderr, "create_game_list: size < 0 !\n");
		exit(-1);
	}

	DBG_MSG(8, "create_game_list: new game list, size %d", size);
	
	gl = (GAME_LIST *) malloc_c(sizeof(GAME_LIST));
	gl->game = (GAME *) malloc_c(sizeof(GAME) * size);
	bzero(gl->game, sizeof(GAME) * size);
	gl->size = size;

	return gl;
}

int connect_to_game(CLIENT *client, char *name) {
	int i, j;
	GAME *game_slot = NULL;

	for (i = 0; game_list != NULL && i < game_list->size; i++) {
		DBG_MSG(10, "connect_to_game: checking slot %d", i);
		if (game_list->game[i].name != NULL) {
			DBG_MSG(9, "connect_to_game: checking game \"%s\"", game_list->game[i].name);
			if (strcmp(game_list->game[i].name, name) == 0) {
				if (!game_list->game[i].is_locked) {
					/* add game to client */
					client->game = &(game_list->game[i]);
					DBG_MSG(8, "connect_to_game: player is in game \"%s\"",
							client->game->name);
					
					/* add client to game */
					for (j = 0; j < game_list->game[i].player_size; j++) {
						if (game_list->game[i].player[j] == NULL) {
							game_list->game[i].player[j] = (struct s_client *)client;
							break;
						}
					}
					return 1;
				} else {
					return 0;
				}
			}
		} else if (game_slot == NULL) {
			DBG_MSG(8, "connect_to_game: have free slot");
			game_slot = &(game_list->game[i]);
		}
	}

	/* game doesn't exist, try to create it */
	if (game_slot) {
		DBG_MSG(6, "connect_to_game: creating game \"%s\"", name);
		game_slot->name = strdup(name);
		game_slot->player = (struct s_client **) malloc_c(cfg.max_connections * sizeof(CLIENT *));
		game_slot->player_size = cfg.max_connections;
		bzero(game_slot->player, cfg.max_connections * sizeof(CLIENT *));
		game_slot->player[0] = (struct s_client *) client;

		/* and connect player */
		client->game = game_slot;
		game_slot->player[0] = (struct s_client *)client;

		DBG_MSG(8, "connect_to_game: player is in game \"%s\"", client->game->name);
		return 2;
	}

	/* there is no place for new game */
	return 0;
}

int send_to_all(GAME *game, char *buf, CLIENT *exclude) {
	int i;
	CLIENT *c = NULL;
	
	if (game && game->player && buf) {
		for (i = 0; i < game->player_size; i++) {
			if ((c = (CLIENT *) game->player[i]) && c->fd != 0 
					&& (!exclude || (CLIENT *)game->player[i] != exclude))
			{
				DBG_MSG(8, "cmd_send_msg: Sending to player \"%s\" (socket %d)...",
						c->name, c->fd);
				write(c->fd, buf, strlen(buf));
			}
		}
		return 1;
	} else {
		return 0;
	}
}

int game_destroy(GAME *game) {
	DBG_MSG(5, "game_destroy");
	if (game->rules) {
		free(game->rules);
		game->rules = NULL;
	}
	if (game->name) {
		free(game->name);
		game->name = NULL;
	}
	if (game->passwd) {
		free(game->passwd);
		game->passwd = NULL;
	}
	if (game->player) {
		free(game->player);
		game->player = NULL;
	}
	return 1;
}

int game_remove_player(CLIENT *client, GAME *game) {
	DBG_MSG(5, "game_remove_player");
	int i, is_zero = 1;
	for (i = 0; game->player && i < game->player_size; i++) {
		if ((CLIENT *) game->player[i] == client) {
			game->player[i] = NULL;
		} else if (game->player[i] != NULL) {
			is_zero = 0;
		}
	}
	if (is_zero) {
		game_destroy(game);
	}
	return 1;
}

int leave_client(CLIENT *client) {
	char *msg;

	DBG_MSG(5, "leave_client (socket %d)", client->fd);

	if (client->game) {
		msg = malloc_c(strlen(client->name) + 10);
		sprintf(msg, "LEAVE %d %s\n", client->fd, client->name);
		send_to_all(client->game, msg, client);
		free(msg);

		game_remove_player(client, client->game);
		client->game = NULL;
	}

	if (client->fd) {
		close_socket(client->fd);
		client->fd = 0;
	}

	if (client->name) {
		free(client->name);
		client->name = NULL;
	}
		
	bzero(client, sizeof(CLIENT));
	return 1;
}

