/*
 * socket.c
 * 
 * Josef Kufner  <jk@myserver.cz>
 *
 * Obsluha serveroveho socketu
 *
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/un.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#define DEBUG 		0
#include "debug.h"

#include "socket.h"

int create_socket(const char *host, const unsigned int port) {
	int server_sockfd, server_len;
	struct sockaddr_in server_address;
	
	/* Tvorba socketu */
	if ((server_sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
		perror("netdesk-server: socket");
		return -1;
	}
	DBG_MSG(6, "create_socket: Socket ok.");	

	/* Konfigurace socketu */
	server_address.sin_family = AF_INET;
	server_address.sin_addr.s_addr = inet_addr(host);
	server_address.sin_port = htons(port);
	server_len = sizeof(server_address);

	/* Pripojeni socketu */
	if (bind(server_sockfd, (struct sockaddr *) &server_address, server_len) == -1) {
		perror("netdesk-server: bind");
		return -1;
	}
	DBG_MSG(6, "create_socket: Bind ok.");	
		

	/* Tvorba fronty */
	if (listen(server_sockfd, 5) == -1) {
		perror("netdesk-server: listen");
		return -1;
	}
	DBG_MSG(6, "create_socket: listen ok.");

	return server_sockfd;
}


int accept_client(int server_sockfd) {
	int client_sockfd, client_len;
	struct sockaddr_in client_address;

	/* Cekani na klienta */	
	client_len = sizeof(client_address);
	if ((client_sockfd = accept(server_sockfd,
			(struct sockaddr *) &client_address, &client_len)) == -1) 
	{
		perror("netdesk-server: accept");
		return -1;
	}
	DBG_MSG(6, "create_socket: Conected.");	
	return client_sockfd;
}


int close_socket(int server_sockfd) {
	fsync(server_sockfd);
	return close(server_sockfd);
}

